package com.king011.fluter.mobile_audio_recorder

import android.media.AudioRecord
import android.os.Handler
import android.os.Looper
import android.os.Process
import android.util.Log
import io.flutter.plugin.common.EventChannel.EventSink
import java.nio.ByteBuffer
import java.nio.ByteOrder

class PcmRecorder : Recorder {
    private val logTag = "MobileAudioRecorderPlugin pcm"
    private var eventSink: EventSink? = null
    private var recording = false
    private var debug = false
    private var record: AudioRecord? = null
    private var audioBuffer: ShortArray? = null
    override fun start() {
        recording = true
        streamData(record!!, audioBuffer!!)
    }

    override fun stop() {
        recording = false
    }

    fun setOptions(debug: Boolean, sampleRate: Int, channel: Int, eventSink: EventSink?): Boolean {
        if (debug) {
            Log.i(logTag, "channel=$channel,sampleRate=$sampleRate")
        }
        // 創建錄音機
        val bufferSize = getBufferSize(sampleRate, channel, eventSink) ?: return false
        val record = createAudioRecord(sampleRate, channel, bufferSize, eventSink)
                ?: return false
        if (record.state != AudioRecord.STATE_INITIALIZED) {
            Log.e(logTag, "Audio Record can't initialize!")
            eventSink!!.error("AudioRecord", "AudioRecord can't initialize!", null)
            return false
        }
        // 爲錄音 分配內存
        audioBuffer = ShortArray(bufferSize / 2)

        this.record = record
        this.debug = debug
        this.eventSink = eventSink
        return true
    }

    private fun streamData(record: AudioRecord, audioBuffer: ShortArray) {
        Thread(Runnable {
            Process.setThreadPriority(Process.THREAD_PRIORITY_AUDIO)

            /** 開始錄音循環 */
            record.startRecording()
            while (recording) {
                val shortOut = record.read(audioBuffer, 0, audioBuffer.size)
                if (shortOut < 1) {
                    continue
                }
                val byteBuffer = ByteBuffer.allocate(shortOut * 2)
                byteBuffer.order(ByteOrder.LITTLE_ENDIAN).asShortBuffer().put(audioBuffer)
                Handler(Looper.getMainLooper()).post {
                    eventSink!!.success(byteBuffer.array())
                }
            }
            record.stop()
            record.release()
        }).start()
    }
}
